# -*- coding: utf-8 -*-

#1a) In molti casi (come p. es. nel caso del DNA mitocondriale umano) il genoma 
# ha una struttura circolare; la sua rappresentazione richiede dunque l'uso di 
# "stringhe circolari", rappresentate attraverso le stringhe usuali ma in cui 
# si assume che il carattere successivo all'ultimo sia il primo, e il precedente
# del primo sia l'ultimo.
# La stessa stringa circolare ha dunque molteplici rappresentazioni come 
# stringa usuale; tra tutte queste, si dice "canonica" la rappresentazione che, 
# tra tutte le possibili, è la prima in ordine lessicografico.
# Si scriva una funzione "canonical_cstr" che, data una rappresentazione di una 
# stringa ciclica, restituisca la corrispondente rappresentazione canonica.
#
# Es: canonical_cstr("GGAACCAATT")      --> AACCAATTGG
#     canonical_cstr("CATGGAACGCAA")    --> AACATGGAACGC
#     canonical_cstr("CTAACCTAACCTAAC") --> AACCTAACCTAACCT


# intuitive solution
def canonical_cstr_naive(target):

    canonical_one=target    
    l0=len(target)
    target += target
    for i in xrange(1,l0):
        checked=target[i:i+l0]
        if checked<canonical_one:
            canonical_one=checked
    return canonical_one


# The following solution is a very efficient one 
# (built according to the so-called "Booth's Algorithm").
def canonical_cstr_booth(target):
    l0=len(target)
    target += target      # Concatenate string to it self to avoid modular arithmetic
    lt=len(target)
    part_match_table = [-1] * lt     # Failure function
    k = 0
    for j in xrange(1,lt): 
        targetj = target[j]
        i = part_match_table[j-k-1]
        while i != -1 and targetj != target[k+i+1]:
            if targetj < target[k+i+1]:
                k = j-i-1
            i = part_match_table[i]
        if targetj != target[k+i+1]:
            if targetj < target[k]:
                k = j
            part_match_table[j-k] = -1
        else:
            part_match_table[j-k] = i+1
    return target[k:k+l0]

#alias for the chosen implementation
canonical_cstr=canonical_cstr_naive
#canonical_cstr=canonical_cstr_booth

